/********************************** (C) COPYRIGHT *******************************
 * File Name          : main.c
 * Author             : WCH
 * Version            : V1.0.0
 * Date               : 2021/06/06
 * Description        : Main program body.
 * Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
 * SPDX-License-Identifier: Apache-2.0
 *******************************************************************************/

/*
 *@Note
  ˫ADCٽ̣
 ADC1ͨ1(PA1),ADC2ͨ1(PA1)),ͨͨADCжϻȡ˫ ADCתݡ

 עCH32V203
*/

#include "debug.h"

/* Global Variable */
u16 Adc_Val[2];
u32 temp;
s16 Calibrattion_Val1 = 0;
s16 Calibrattion_Val2 = 0;

u32 l = 0;

#define            GENERAL_TIM                   TIM2
#define            GENERAL_TIM_APBxClock_FUN     RCC_APB1PeriphClockCmd
#define            GENERAL_TIM_CLK               RCC_APB1Periph_TIM2
#define            GENERAL_TIM_Period            (1000-1)
#define            GENERAL_TIM_Prescaler         35
#define            GENERAL_TIM_IRQ               TIM2_IRQn
#define            GENERAL_TIM_IRQHandler        TIM2_IRQHandler

void ADC1_2_IRQHandler  (void) __attribute__((interrupt()));
void TIM2_IRQHandler    (void) __attribute__((interrupt()));
/*********************************************************************
 * @fn      ADC_Function_Init
 *
 * @brief   Initializes ADC collection.
 *
 * @return  none
 */
void ADC_Function_Init(void)
{
    ADC_InitTypeDef  ADC_InitStructure = {0};
    GPIO_InitTypeDef GPIO_InitStructure = {0};
    NVIC_InitTypeDef NVIC_InitStructure = {0};

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC2, ENABLE);
    RCC_ADCCLKConfig(RCC_PCLK2_Div6);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    ADC_DeInit(ADC1);
    ADC_DeInit(ADC2);

    NVIC_InitStructure.NVIC_IRQChannel = ADC1_2_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 2;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);

    ADC_InitStructure.ADC_Mode = ADC_Mode_FastInterl;
    ADC_InitStructure.ADC_ScanConvMode = DISABLE;
    ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;
    ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;
    ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
    ADC_InitStructure.ADC_NbrOfChannel = 1;
    ADC_InitStructure.ADC_OutputBuffer = ADC_OutputBuffer_Disable;
    ADC_InitStructure.ADC_Pga = ADC_Pga_1;

    ADC_Init(ADC1, &ADC_InitStructure);
    ADC_RegularChannelConfig(ADC1, ADC_Channel_1, 1, ADC_SampleTime_1Cycles5);

    ADC_ITConfig(ADC1, ADC_IT_EOC, ENABLE);
    ADC_DMACmd(ADC1, ENABLE);
    ADC_Cmd(ADC1, ENABLE);

    ADC_BufferCmd(ADC1, DISABLE); //disable buffer
    ADC_ResetCalibration(ADC1);
    while(ADC_GetResetCalibrationStatus(ADC1));
    ADC_StartCalibration(ADC1);
    while(ADC_GetCalibrationStatus(ADC1));
    Calibrattion_Val1 = Get_CalibrationValue(ADC1);

    ADC_BufferCmd(ADC1, ENABLE); //enable buffer

    ADC_Init(ADC2, &ADC_InitStructure);
    ADC_RegularChannelConfig(ADC2, ADC_Channel_1, 1, ADC_SampleTime_1Cycles5);

    ADC_ITConfig(ADC2, ADC_IT_EOC, ENABLE);
    ADC_SoftwareStartConvCmd(ADC2, ENABLE);
    ADC_Cmd(ADC2, ENABLE);

    ADC_BufferCmd(ADC2, DISABLE); //disable buffer
    ADC_ResetCalibration(ADC2);
    while(ADC_GetResetCalibrationStatus(ADC2));
    ADC_StartCalibration(ADC2);
    while(ADC_GetCalibrationStatus(ADC2));
    Calibrattion_Val2 = Get_CalibrationValue(ADC2);

    ADC_BufferCmd(ADC2, ENABLE); //enable buffer
}

/*********************************************************************
 * @fn      Get_ConversionVal1
 *
 * @brief   Get Conversion Value.
 *
 * @param   val - Sampling value
 *
 * @return  val+Calibrattion_Val - Conversion Value.
 */
u16 Get_ConversionVal1(s16 val)
{
    if((val + Calibrattion_Val1) < 0)
        return 0;
    if((Calibrattion_Val1 + val) > 4095||val==4095)
        return 4095;
    return (val + Calibrattion_Val1);
}

/*********************************************************************
 * @fn      Get_ConversionVal2
 *
 * @brief   Get Conversion Value.
 *
 * @param   val - Sampling value
 *
 * @return  val+Calibrattion_Val - Conversion Value.
 */
u16 Get_ConversionVal2(s16 val)
{
    if((val + Calibrattion_Val2) < 0)
        return 0;
    if((Calibrattion_Val2 + val) > 4095||val==4095)
        return 4095;
    return (val + Calibrattion_Val2);
}

static void GENERAL_TIM_NVIC_Config(void)
{
    NVIC_InitTypeDef NVIC_InitStructure;
    // жΪ0
//    NVIC_PriorityGroupConfig(NVIC_PriorityGroup_0);
    // жԴ
    NVIC_InitStructure.NVIC_IRQChannel = GENERAL_TIM_IRQ ;
    // ȼΪ1
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
    // ռȼΪ0
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);
}

static void GENERAL_TIM_Mode_Config(void)
{
    TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
    // ʱʱ,ڲʱCK_INT=72M
    GENERAL_TIM_APBxClock_FUN(GENERAL_TIM_CLK, ENABLE);
    // ԶװؼĴֵۼTIM_Period+1Ƶʺһ»ж
    TIM_TimeBaseStructure.TIM_Period=GENERAL_TIM_Period;
    // ʱԤƵ
    TIM_TimeBaseStructure.TIM_Prescaler= GENERAL_TIM_Prescaler;
    // ʱӷƵ ûõù
    TIM_TimeBaseStructure.TIM_ClockDivision=TIM_CKD_DIV1;
    // ģʽΪϼ
    TIM_TimeBaseStructure.TIM_CounterMode=TIM_CounterMode_Up;
    // ظֵûõù
    TIM_TimeBaseStructure.TIM_RepetitionCounter=0;
    // ʼʱ
    TIM_TimeBaseInit(GENERAL_TIM, &TIM_TimeBaseStructure);

    // жϱ־λ
    TIM_ClearFlag(GENERAL_TIM, TIM_FLAG_Update);

    // ж
    TIM_ITConfig(GENERAL_TIM,TIM_IT_Update,ENABLE);

    // ʹܼ
    TIM_Cmd(GENERAL_TIM, ENABLE);
}

void GENERAL_TIM_Init(void)
{
    GENERAL_TIM_NVIC_Config();
    GENERAL_TIM_Mode_Config();
}

void GPIO_Toggle_INIT(void)
{
    GPIO_InitTypeDef GPIO_InitStructure = {0};

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
}

/*********************************************************************
 * @fn      main
 *
 * @brief   Main program.
 *
 * @return  none
 */
int main(void)
{
    NVIC_PriorityGroupConfig(NVIC_PriorityGroup_0);
    Delay_Init();
    USART_Printf_Init(115200);
    printf("SystemClk:%d\r\n", SystemCoreClock);

    ADC_Function_Init();
    printf("CalibrattionValue1:%d\r\n", Calibrattion_Val1);
    printf("CalibrattionValue2:%d\r\n", Calibrattion_Val2);

    GENERAL_TIM_Init();
    GPIO_Toggle_INIT();

    while(1)
    {
//        ADC_SoftwareStartConvCmd(ADC1, ENABLE);

//        Delay_Ms(50);

//        printf("l --> %d\r\n", l);
    }
}

/*********************************************************************
 * @fn      ADC1_2_IRQHandler
 *
 * @brief   ADC1_2 Interrupt Service Function.
 *
 * @return  none
 */
void ADC1_2_IRQHandler()
{
    if(ADC_GetITStatus(ADC1, ADC_IT_EOC))
    {
        temp = ADC1->RDATAR;
        Adc_Val[0] = temp & 0xffff;
        Adc_Val[1] = (temp >> 16) & 0xffff;

        printf("ADC2 ch2=%d\r\n", Get_ConversionVal2(Adc_Val[1]));
        printf("ADC1 ch2=%d\r\n", Get_ConversionVal1(Adc_Val[0]));

        ADC_ClearITPendingBit(ADC1, ADC_IT_EOC);
        ADC_ClearITPendingBit(ADC2, ADC_IT_EOC);
    }

}

void  TIM2_IRQHandler (void)
{
    if ( TIM_GetITStatus( GENERAL_TIM, TIM_IT_Update) != RESET )
    {
//        l++;
//        GPIO_WriteBit(GPIOA, GPIO_Pin_0, (l == 0) ? (l = Bit_SET) : (l = Bit_RESET));
        ADC_SoftwareStartConvCmd(ADC1, ENABLE);
        TIM_ClearITPendingBit(GENERAL_TIM , TIM_FLAG_Update);
    }
}


